package com.unis.ynls.dataDictionary.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：Redis工具类
 *
 * @author 杨祖榕
 * @version v1.0
 * @date 2018/11/20 16:10
 */
@Component
public class RedisUtil<T extends Serializable> {

    private final int HALF_HOUR = 1800;
    private final int ONE_HOUR = 360000000;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 将List以Json格式存入Redis
     * @param key
     * @param list
     */
    public void setListDataByJson(String key, List<T> list) {
        delete(key);
        redisTemplate.opsForValue().set(key, JSONArray.toJSONString(list), ONE_HOUR, TimeUnit.SECONDS);
    }


    /**
     * 获取List对象(该对象原先以Json格式存储)
     * @param key
     * @return
     */
    public List<T> getListDataByJson(String key, Class<T> tClass) {
        Object obj = redisTemplate.opsForValue().get(key);
        if(null != obj) {
            String str = obj.toString();
            List list = JSONObject.parseArray(str, tClass);
            return list;
        }
        return null;
    }

    /**
     * 将Set以Json格式存入Redis
     * @param key
     * @param setData
     */
    public void setSetDataByJson(String key, Set<T> setData) {
        if(null != setData) {
            List<T> list = new ArrayList<T>(setData);
            setListDataByJson(key, list);
        }
    }

    /**
     * 获取Set对象(该对象原先以Json格式存储)
     * @param key
     * @return
     */
    public Set<T> getSetDataByJson(String key, Class<T> tClass) {
        List<T> list = getListDataByJson(key, tClass);
        if(null != list && list.size() > 0){
            return new HashSet<T>(list);
        }
        return null;
    }

    /**
     * 将Map以Json格式存入Redis
     * @param key
     * @param map
     */
    public void setMapDataByJson(String key, Map<String, T> map) {
        delete(key);
        redisTemplate.opsForValue().set(key, JSON.toJSONString(map), ONE_HOUR, TimeUnit.SECONDS);
    }

    /**
     * 获取Map对象(该对象原先以Json格式存储)
     * @param key
     * @return
     */
    public Map<String, T> getMapDataByJson(String key, Class<T> tClass) {
        Map<String, Object> tMap = getMapDataByJson(key);

        if(null != tMap && tMap.size() > 0) {
            Set<String> keySet = tMap.keySet();
            Map<String, T> map = new HashMap<>();
            for(String tKey : keySet) {
                map.put(tKey, JSONObject.parseObject(tMap.get(tKey).toString(), tClass));
            }
            return  map;
        }

        return null;
    }

    public Map<String, Object> getMapDataByJson(String key) {
        Object obj = redisTemplate.opsForValue().get(key);
        if(null != obj) {
            String str = obj.toString();
            return JSONObject.parseObject(str, Map.class);
        }
        return null;
    }

    /**
     * 将某类型对象以Json格式存入Redis
     * @param key
     * @param t
     */
    public void setObjectByJson(String key, T t) {
        delete(key);
        redisTemplate.opsForValue().set(key, JSONObject.toJSON(t), ONE_HOUR, TimeUnit.SECONDS);
    }

    /**
     * 获取某类型对象(该对象原先以Json格式存储)
     * @param key
     * @param tClass
     * @return
     */
    public T getObjectByJson(String key, Class<T> tClass) {
        Object obj = redisTemplate.opsForValue().get(key);
        if(null != obj) {
            String str = obj.toString();
            T t = JSONObject.parseObject(str, tClass);
            return t;
        }
        return null;
    }


    /**
     * 删除Redis上某key的数据
     * @param key
     */
    public void delete(String key) {
        redisTemplate.delete(key);
    }

    /**
     * 清空
     *
     * @param key
     */
    public void clearList(String key) {
        while (redisTemplate.opsForList().size(key) > 0) {
            redisTemplate.opsForList().leftPop(key);
        }
    }

//    public Long batchDelete(Set keySet) {
//        Long count = 0L;
//        if(null != keySet && keySet.size() > 0){
//            count = redisTemplate.delete(keySet);
//        }
//        return count;
//    }
//
//    public Set<String> getKeysByWildcard(String keys) {
//        Set<String> keySet = redisTemplate.keys(keys);
//        return keySet;
//    }



/////////////////////////////////////////////////////////////////////////////////
    //处理

    /**
     * 给一个指定的 key 值附加过期时间
     *
     * @param key  the key
     * @param time the time
     * @param type the type
     * @return boolean
     */
    public boolean expire(String key, long time, TimeUnit type) {
        return redisTemplate.boundValueOps( key ).expire( time, type );
    }

    /**
     * 移除指定key 的过期时间
     *
     * @param key the key
     * @return boolean
     */
    public boolean persist(String key) {
        return redisTemplate.boundValueOps( key ).persist();
    }

    /**
     * 获取指定key 的过期时间
     *
     * @param key the key
     * @return expire
     */
    public Long getExpire(String key) {
        return redisTemplate.boundValueOps( key ).getExpire();
    }

    /**
     * 修改 key的名称
     *
     * @param key    the key
     * @param newKey the new key
     * @return
     */
    public void rename(String key, String newKey) {
        redisTemplate.boundValueOps( key ).rename( newKey );
    }


    /**
     * 删除指定 hash 的 HashKey
     *
     * @param key
     * @param hashKeys
     * @return 删除成功的 数量
     */
    public Long deleteHash(String key, String... hashKeys) {
        return redisTemplate.opsForHash().delete( key, hashKeys );
    }



    /**
     * 设置 String 类型 key-value
     *
     * @param key   the key
     * @param value the value
     */
    public void setStr(String key, String value) {
    	boolean absentBoolean = redisTemplate.opsForValue().setIfAbsent(key,value);
    	if(!absentBoolean){
	    	redisTemplate.opsForValue().getAndSet(key, value);
    	 }
    }


    /**
     * 获取 String 类型 key-value
     *
     * @param key the key
     * @return string
     */
    public String getStr(String key) {
        return (String) redisTemplate.opsForValue().get( key );
    }



    /**
     * 设置 String 类型 key-value 并添加过期时间
     * @param key   the key
     * @param value the value
     * @param time  过期时间
     * @param type  the type
     */
    public void setForTime(String key, String value, long time, TimeUnit type) {
        redisTemplate.opsForValue().set( key, value, time, type );
    }


    /**
     * 批量添加 key-value (重复的键会覆盖)
     *
     * @param keyAndValue the key and value
     */
    public void batchSetStr(Map<String, String> keyAndValue) {
        redisTemplate.opsForValue().multiSet( keyAndValue );
    }



    /**
     * 添加 Hash 键值对
     *
     * @param key     the key
     * @param hashKey the hash key
     * @param value   the value
     */
    public void putHash(RedisTemplate redisTemplate, String key, String hashKey, String value) {
        redisTemplate.opsForHash().put( key, hashKey, value );
    }


    /**
     * 批量添加 hash 的 键值对
     * 有则覆盖,没有则添加
     *
     * @param key
     * @param map
     */
    public void putAllHash(RedisTemplate redisTemplate, String key, Map <String, String> map) {
        redisTemplate.opsForHash().putAll( key, map );
    }



    /**
     * 给指定 hash 的 hashkey 做增减操作
     *
     * @param key     the key
     * @param hashKey the hash key
     * @param number  the number
     * @return long
     */
    public Long incrementHash(RedisTemplate redisTemplate,String key, String hashKey, long number) {
        return redisTemplate.opsForHash().increment( key, hashKey, number );
    }

    /**
     * 给指定 hash 的 hashkey 做增减操作
     *
     * @param key     the key
     * @param hashKey the hash key
     * @param number  the number
     * @return double
     */
    public Double incrementHash(RedisTemplate redisTemplate, String key, String hashKey, Double number) {
        return redisTemplate.opsForHash().increment( key, hashKey, number );
    }

    /**
     * 获取指定 key 下的 hashkey
     *
     * @param key     the key
     * @param hashKey the hash key
     * @return hash key
     */
    public Object getHashKey(RedisTemplate redisTemplate, String key, String hashKey) {
        return redisTemplate.opsForHash().get( key, hashKey );
    }


    /**
     * 获取 key 下的 所有  hashkey 和 value
     *
     * @param key
     * @return
     */
    public Map <Object, Object> getHashEntries(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForHash().entries( key );
    }

    /**
     * 验证指定 key 下 有没有指定的 hashkey
     *
     * @param key
     * @param hashKey
     * @return
     */
    public boolean hashKey(RedisTemplate redisTemplate, String key, String hashKey) {
        return redisTemplate.opsForHash().hasKey( key, hashKey );
    }

    /**
     * 获取 key 下的 所有 hashkey 字段名
     *
     * @param key
     * @return
     */
    public Set<Object> hashKeys(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForHash().keys( key );
    }


    /**
     * 获取指定 hash 下面的 键值对 数量
     *
     * @param key
     * @return
     */
    public Long hashSize(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForHash().size( key );
    }

    //List 操作

    /**
     * 指定 list 从左入栈
     *
     * @param key
     * @return 当前队列的长度
     */
    public Long leftPushList(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForList().leftPush( key, value );
    }

    /**
     * 指定 list 从左出栈
     * 如果列表没有元素,会堵塞到列表一直有元素或者超时为止
     *
     * @param key
     * @return 出栈的值
     */
    public Object leftPopList(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForList().leftPop( key );
    }

    /**
     * 从左边依次入栈
     * 导入顺序按照 Collection 顺序
     * 如: a b c => c b a
     *
     * @param key
     * @param values
     * @return
     */
    public Long leftPushAllList(RedisTemplate redisTemplate, String key, Collection<Object> values) {
        return redisTemplate.opsForList().leftPushAll( key, values );
    }

    /**
     * 指定 list 从右入栈
     *
     * @param key
     * @return 当前队列的长度
     */
    public Long rightPushList(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForList().rightPush( key, value );
    }

    /**
     * 指定 list 从右出栈
     * 如果列表没有元素,会堵塞到列表一直有元素或者超时为止
     *
     * @param key
     * @return 出栈的值
     */
    public Object rightPopList(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForList().rightPop( key );
    }

    /**
     * 从右边依次入栈
     * 导入顺序按照 Collection 顺序
     * 如: a b c => a b c
     *
     * @param key
     * @param values
     * @return
     */
    public Long rightPushAllList(RedisTemplate redisTemplate, String key, Collection <Object> values) {
        return redisTemplate.opsForList().rightPushAll( key, values );
    }


    /**
     * 根据下标获取值
     *
     * @param key
     * @param index
     * @return
     */
    public Object popIndexList(RedisTemplate redisTemplate, String key, long index) {
        return redisTemplate.opsForList().index( key, index );
    }


    /**
     * 获取列表指定长度
     *
     * @param key
     * @param index
     * @return
     */
    public Long listSize(RedisTemplate redisTemplate, String key, long index) {
        return redisTemplate.opsForList().size( key );
    }


    /**
     * 获取列表 指定范围内的所有值
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public List <Object> listRange(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForList().range( key, start, end );
    }


    /**
     * 删除 key 中 值为 value 的 count 个数.
     *
     * @param key
     * @param count
     * @param value
     * @return 成功删除的个数
     */
    public Long listRemove(RedisTemplate redisTemplate, String key, long count, Object value) {
        return redisTemplate.opsForList().remove( key, count, value );
    }


    /**
     * 删除 列表 [start,end] 以外的所有元素
     *
     * @param key
     * @param start
     * @param end
     */
    public void listTrim(RedisTemplate redisTemplate, String key, long start, long end) {
        redisTemplate.opsForList().trim( key, start, end );

    }

    /**
     * 将 key 右出栈,并左入栈到 key2
     *
     * @param key  右出栈的列表
     * @param key2 左入栈的列表
     * @return 操作的值
     */
    public Object rightPopAndLeftPushList(RedisTemplate redisTemplate, String key, String key2) {
        return redisTemplate.opsForList().rightPopAndLeftPush( key, key2 );
    }

    //set 操作  无序不重复集合

    /**
     * 添加 set 元素
     *
     * @param key
     * @param values
     * @return
     */
    public Long addSet(RedisTemplate redisTemplate, String key, String... values) {
        return redisTemplate.opsForSet().add( key, values );
    }

    /**
     * 获取两个集合的差集
     *
     * @param key      the key
     * @param otherkey the otherkey
     * @return set
     */
    public Set <Object> differenceSet(RedisTemplate redisTemplate, String key, String otherkey) {
        return redisTemplate.opsForSet().difference( key, otherkey );
    }


    /**
     * 获取 key 和 集合  collections 中的 key 集合的差集
     *
     * @param key       the key
     * @param otherKeys the other keys
     * @return set
     */
    public Set <Object> differenceSet(RedisTemplate redisTemplate, String key, Collection <Object> otherKeys) {
        return redisTemplate.opsForSet().difference( key, otherKeys );
    }

    /**
     * 将  key 与 otherkey 的差集 ,添加到新的 newKey 集合中
     *
     * @param key
     * @param otherkey
     * @param newKey
     * @return 返回差集的数量
     */
    public Long differenceAndStoreSet(RedisTemplate redisTemplate, String key, String otherkey, String newKey) {
        return redisTemplate.opsForSet().differenceAndStore( key, otherkey, newKey );
    }

    /**
     * 将 key 和 集合  collections 中的 key 集合的差集 添加到  newkey 集合中
     *
     * @param key
     * @param otherKeys
     * @param newKey
     * @return 返回差集的数量
     */
    public Long differenceAndStoreSet(RedisTemplate redisTemplate, String key, Collection <Object> otherKeys, String newKey) {
        return redisTemplate.opsForSet().differenceAndStore( newKey, otherKeys, newKey );
    }

    /**
     * 删除一个或多个集合中的指定值
     *
     * @param key
     * @param values
     * @return 成功删除数量
     */
    public Long removeSet(RedisTemplate redisTemplate, String key, Object... values) {
        return redisTemplate.opsForSet().remove( key, values );
    }

    /**
     * 随机移除一个元素,并返回出来
     *
     * @param key
     * @return
     */
    public Object randomSetPop(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForSet().pop( key );
    }

    /**
     * 随机获取一个元素
     *
     * @param key
     * @return
     */
    public Object randomSet(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForSet().randomMember( key );
    }

    /**
     * 随机获取指定数量的元素,同一个元素可能会选中两次
     *
     * @param key
     * @param count
     * @return
     */
    public List <Object> randomSet(RedisTemplate redisTemplate, String key, long count) {
        return redisTemplate.opsForSet().randomMembers( key, count );
    }

    /**
     * 随机获取指定数量的元素,去重(同一个元素只能选择两一次)
     *
     * @param key
     * @param count
     * @return
     */
    public Set <Object> randomSetDistinctSet(RedisTemplate redisTemplate, String key, long count) {
        return redisTemplate.opsForSet().distinctRandomMembers( key, count );
    }

    /**
     * 将 key 中的 value 转入到 destKey 中
     *
     * @param key
     * @param value
     * @param destKey
     * @return 返回成功与否
     */
    public boolean moveSet(RedisTemplate redisTemplate, String key, Object value, String destKey) {
        return redisTemplate.opsForSet().move( key, value, destKey );
    }

    /**
     * 无序集合的大小
     *
     * @param key
     * @return
     */
    public Long setSizeSet(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForSet().size( key );
    }

    /**
     * 判断 set 集合中 是否有 value
     *
     * @param key
     * @param value
     * @return
     */
    public boolean isMemberSet(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForSet().isMember( key, value );
    }

    /**
     * 返回 key 和 othere 的并集
     *
     * @param key
     * @param otherKey
     * @return
     */
    public Set <Object> unionSet(RedisTemplate redisTemplate, String key, String otherKey) {
        return redisTemplate.opsForSet().union( key, otherKey );
    }

    /**
     * 返回 key 和 otherKeys 的并集
     *
     * @param key       the key
     * @param otherKeys the other keys
     * @return set
     */
    public Set <Object> unionSet(RedisTemplate redisTemplate, String key, Collection <Object> otherKeys) {
        return redisTemplate.opsForSet().union( key, otherKeys );
    }

    /**
     * 将 key 与 otherKey 的并集,保存到 destKey 中
     *
     * @param key
     * @param otherKey
     * @param destKey
     * @return destKey 数量
     */
    public Long unionAndStoreSet(RedisTemplate redisTemplate, String key, String otherKey, String destKey) {
        return redisTemplate.opsForSet().unionAndStore( key, otherKey, destKey );
    }

    /**
     * 将 key 与 otherKey 的并集,保存到 destKey 中
     *
     * @param key
     * @param otherKeys
     * @param destKey
     * @return destKey 数量
     */
    public Long unionAndStoreSet(RedisTemplate redisTemplate, String key, Collection <Object> otherKeys, String destKey) {
        return redisTemplate.opsForSet().unionAndStore( key, otherKeys, destKey );
    }

    /**
     * 返回集合中所有元素
     *
     * @param key
     * @return
     */
    public Set <Object> membersSet(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForSet().members( key );
    }

    //Zset 根据 socre 排序   不重复 每个元素附加一个 socre  double类型的属性(double 可以重复)

    /**
     * 添加 ZSet 元素
     *
     * @param key
     * @param value
     * @param score
     */
    public boolean addZSet(RedisTemplate redisTemplate, String key, Object value, double score) {
        return redisTemplate.opsForZSet().add( key, value, score );
    }

    /**
     * 批量添加 Zset <br>
     * Set<TypedTuple<Object>> tuples = new HashSet<>();<br>
     * TypedTuple<Object> objectTypedTuple1 = new DefaultTypedTuple<Object>("zset-5",9.6);<br>
     * tuples.add(objectTypedTuple1);
     *
     * @param key
     * @param tuples
     * @return
     */
    public Long batchAddZSet(RedisTemplate redisTemplate, String key, Set <ZSetOperations.TypedTuple <Object>> tuples) {
        return redisTemplate.opsForZSet().add( key, tuples );
    }

    /**
     * Zset 删除一个或多个元素
     *
     * @param key
     * @param values
     * @return
     */
    public Long removeZSet(RedisTemplate redisTemplate, String key, String... values) {
        return redisTemplate.opsForZSet().remove( key, values );
    }

    /**
     * 对指定的 zset 的 value 值 , socre 属性做增减操作
     *
     * @param key
     * @param value
     * @param score
     * @return
     */
    public Double incrementScoreZSet(RedisTemplate redisTemplate, String key, Object value, double score) {
        return redisTemplate.opsForZSet().incrementScore( key, value, score );
    }

    /**
     * 获取 key 中指定 value 的排名(从0开始,从小到大排序)
     *
     * @param key
     * @param value
     * @return
     */
    public Long rankZSet(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForZSet().rank( key, value );
    }

    /**
     * 获取 key 中指定 value 的排名(从0开始,从大到小排序)
     *
     * @param key
     * @param value
     * @return
     */
    public Long reverseRankZSet(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForZSet().reverseRank( key, value );
    }

    /**
     * 获取索引区间内的排序结果集合(从0开始,从小到大,带上分数)
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> rangeWithScoresZSet(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForZSet().rangeWithScores( key, start, end );
    }

    /**
     * 获取索引区间内的排序结果集合(从0开始,从小到大,只有列名)
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public Set <Object> rangeZSet(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForZSet().range( key, start, end );
    }

    /**
     * 获取分数范围内的 [min,max] 的排序结果集合 (从小到大,只有列名)
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public Set <Object> rangeByScoreZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().rangeByScore( key, min, max );
    }

    /**
     * 获取分数范围内的 [min,max] 的排序结果集合 (从小到大,集合带分数)
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> rangeByScoreWithScoresZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().rangeByScoreWithScores( key, min, max );
    }

    /**
     * 返回 分数范围内 指定 count 数量的元素集合, 并且从 offset 下标开始(从小到大,不带分数的集合)
     *
     * @param key
     * @param min
     * @param max
     * @param offset 从指定下标开始
     * @param count  输出指定元素数量
     * @return
     */
    public Set <Object> rangeByScoreZSet(RedisTemplate redisTemplate, String key, double min, double max, long offset, long count) {
        return redisTemplate.opsForZSet().rangeByScore( key, min, max, offset, count );
    }

    /**
     * 返回 分数范围内 指定 count 数量的元素集合, 并且从 offset 下标开始(从小到大,带分数的集合)
     *
     * @param key
     * @param min
     * @param max
     * @param offset 从指定下标开始
     * @param count  输出指定元素数量
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> rangeByScoreWithScoresZSet(RedisTemplate redisTemplate, String key, double min, double max, long offset, long count) {
        return redisTemplate.opsForZSet().rangeByScoreWithScores( key, min, max, offset, count );
    }

    /**
     * 获取索引区间内的排序结果集合(从0开始,从大到小,只有列名)
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public Set <Object> reverseRangeZSet(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForZSet().reverseRange( key, start, end );
    }

    /**
     * 获取索引区间内的排序结果集合(从0开始,从大到小,带上分数)
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> reverseRangeWithScoresZSet(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForZSet().reverseRangeWithScores( key, start, end );
    }

    /**
     * 获取分数范围内的 [min,max] 的排序结果集合 (从大到小,集合不带分数)
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public Set <Object> reverseRangeByScoreZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().reverseRangeByScore( key, min, max );
    }

    /**
     * 获取分数范围内的 [min,max] 的排序结果集合 (从大到小,集合带分数)
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> reverseRangeByScoreWithScoresZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().reverseRangeByScoreWithScores( key, min, max );
    }

    /**
     * 返回 分数范围内 指定 count 数量的元素集合, 并且从 offset 下标开始(从大到小,不带分数的集合)
     *
     * @param key
     * @param min
     * @param max
     * @param offset 从指定下标开始
     * @param count  输出指定元素数量
     * @return
     */
    public Set <Object> reverseRangeByScoreZSet(RedisTemplate redisTemplate, String key, double min, double max, long offset, long count) {
        return redisTemplate.opsForZSet().reverseRangeByScore( key, min, max, offset, count );
    }

    /**
     * 返回 分数范围内 指定 count 数量的元素集合, 并且从 offset 下标开始(从大到小,带分数的集合)
     *
     * @param key
     * @param min
     * @param max
     * @param offset 从指定下标开始
     * @param count  输出指定元素数量
     * @return
     */
    public Set <ZSetOperations.TypedTuple <Object>> reverseRangeByScoreWithScoresZSet(RedisTemplate redisTemplate, String key, double min, double max, long offset, long count) {
        return redisTemplate.opsForZSet().reverseRangeByScoreWithScores( key, min, max, offset, count );
    }

    /**
     * 返回指定分数区间 [min,max] 的元素个数
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public long countZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().count( key, min, max );
    }

    /**
     * 返回 zset 集合数量
     *
     * @param key
     * @return
     */
    public long sizeZSet(RedisTemplate redisTemplate, String key) {
        return redisTemplate.opsForZSet().size( key );
    }

    /**
     * 获取指定成员的 score 值
     *
     * @param key
     * @param value
     * @return
     */
    public Double scoreZSet(RedisTemplate redisTemplate, String key, Object value) {
        return redisTemplate.opsForZSet().score( key, value );
    }

    /**
     * 删除指定索引位置的成员,其中成员分数按( 从小到大 )
     *
     * @param key
     * @param start
     * @param end
     * @return
     */
    public Long removeRangeZSet(RedisTemplate redisTemplate, String key, long start, long end) {
        return redisTemplate.opsForZSet().removeRange( key, start, end );
    }

    /**
     * 删除指定 分数范围 内的成员 [main,max],其中成员分数按( 从小到大 )
     *
     * @param key
     * @param min
     * @param max
     * @return
     */
    public Long removeRangeByScoreZSet(RedisTemplate redisTemplate, String key, double min, double max) {
        return redisTemplate.opsForZSet().removeRangeByScore( key, min, max );
    }

    /**
     * key 和 other 两个集合的并集,保存在 destKey 集合中, 列名相同的 score 相加
     *
     * @param key
     * @param otherKey
     * @param destKey
     * @return
     */
    public Long unionAndStoreZSet(RedisTemplate redisTemplate, String key, String otherKey, String destKey) {
        return redisTemplate.opsForZSet().unionAndStore( key, otherKey, destKey );
    }

    /**
     * key 和 otherKeys 多个集合的并集,保存在 destKey 集合中, 列名相同的 score 相加
     *
     * @param key
     * @param otherKeys
     * @param destKey
     * @return
     */
    public Long unionAndStoreZSet(RedisTemplate redisTemplate, String key, Collection <String> otherKeys, String destKey) {
        return redisTemplate.opsForZSet().unionAndStore( key, otherKeys, destKey );
    }

    /**
     * key 和 otherKey 两个集合的交集,保存在 destKey 集合中
     *
     * @param key
     * @param otherKey
     * @param destKey
     * @return
     */
    public Long intersectAndStoreZSet(RedisTemplate redisTemplate, String key, String otherKey, String destKey) {
        return redisTemplate.opsForZSet().intersectAndStore( key, otherKey, destKey );
    }

    /**
     * key 和 otherKeys 多个集合的交集,保存在 destKey 集合中
     *
     * @param key
     * @param otherKeys
     * @param destKey
     * @return
     */
    public Long intersectAndStoreZSet(RedisTemplate redisTemplate, String key, Collection <String> otherKeys, String destKey) {
        return redisTemplate.opsForZSet().intersectAndStore( key, otherKeys, destKey );
    }

    public <T> ValueOperations<String, T> setCacheObject(String key, T value) {
        ValueOperations<String, T> operation = redisTemplate.opsForValue();
        operation.set(key, value);
        return operation;
    }
    public <T> T getCacheObject(String key) {
        ValueOperations<String, T> operation = redisTemplate.opsForValue();
        return operation.get(key);
    }

    /**
     * 切换数据库
     * @param index
     */
    public void changeData(int index) {
        LettuceConnectionFactory factory = (LettuceConnectionFactory) redisTemplate.getConnectionFactory();
        factory.setDatabase(index);
        redisTemplate.setConnectionFactory(factory);
    }

}
