package com.unis.ynls.dataDictionary.utils.jwt;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.tomcat.util.codec.binary.Base64;

import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @ClassName: JwtUtil
 * @Description: jwt工具类
 * @author tiechanglong
 * @date 2018年11月22日 上午10:34:04
 * 
 * @version 1.2.0
 */
public class JwtUtil {

	public static String USER_ID = "userId";
	public static String USER_ACCOUNT = "userAccount";
	public static Long CREAT_TIME;
	
	/**
	 * @Description: 生成加密串
	 *
	 * @date 2018年11月22日,下午2:07:23
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @return
	 */
	public static SecretKey generalKey() {
		String stringKey = JwtConstant.JWT_SECRET;
		// 本地的密码解码
		byte[] encodedKey = Base64.decodeBase64(stringKey);
		// 根据给定的字节数组使用AES加密算法构造一个密钥
		SecretKey key = new SecretKeySpec(encodedKey, 0, encodedKey.length, "AES");
		return key;
	}

	/**
	 * 
	 * @Description: 创建jwt
	 *
	 * @date 2018年11月22日,下午2:08:59
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param id
	 * @param issuer
	 * @param subject
	 * @param ttlMillis
	 * @return
	 * @throws Exception
	 */
	public static String createJWT(String id, String userId, String userAccount, String issuer, String subject, long ttlMillis){

		// 指定签名的时候使用的签名算法，也就是header那部分，jjwt已经将这部分内容封装好了。
		SignatureAlgorithm signatureAlgorithm = SignatureAlgorithm.HS256;

		// 生成JWT的时间
		long nowMillis = System.currentTimeMillis();
		CREAT_TIME = nowMillis;
		Date now = new Date(CREAT_TIME);

		// 创建payload的私有声明
		Map<String, Object> claims = new HashMap<>();
		claims.put(USER_ID, userId);
		claims.put(USER_ACCOUNT, userAccount);

		// 生成签名的时候使用的秘钥secret，一旦客户端得知这个secret, 那就意味着客户端是可以自我签发jwt了。
		SecretKey key = generalKey();

		// 下面就是在为payload添加各种标准声明和私有声明了
		JwtBuilder builder = Jwts.builder()
				.setClaims(claims)
				.setId(id)
				.setIssuedAt(now)
				.setSubject(subject)
				.signWith(signatureAlgorithm, key);

		// 设置过期时间
		if (ttlMillis >= 0) {
			long expMillis = nowMillis + ttlMillis;
			Date exp = new Date(expMillis);
			builder.setExpiration(exp);
		}
		return builder.compact();
	}

	/**
	 *
	 * @Description: 解析JWT
	 *
	 * @date 2018年11月22日,下午2:14:37
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return
	 * @throws Exception
	 */
	public static Claims parseJWT(String jwt) {
		SecretKey key = generalKey();
		// 得到DefaultJwtParser
		Claims claims = Jwts.parser()
				// 设置签名的秘钥
				.setSigningKey(key)
				.parseClaimsJws(jwt)
				.getBody(); // 设置需要解析的jwt
		return claims;
	}

	/**
	 * 
	 * @Description: 获取jwtID
	 *
	 * @date 2018年12月1日,下午3:05:11
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return
	 */
	public static String getJwtId(String jwt) {
		String jwtId;
		Claims parseJWT = parseJWT(jwt);
		jwtId = parseJWT.getId();
		return jwtId;
	}
	/**
	 * 
	 * @Description: 获取用户ID
	 *
	 * @date 2018年12月1日,下午3:16:27
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return
	 */
	public static String getUserId(String jwt) {
		Claims parseJWT = parseJWT(jwt);
		String userId = parseJWT.get(USER_ID, String.class);
		return userId;
	}
	/**
	 * 
	 * @Description: 获取用户名
	 *
	 * @date 2018年12月1日,下午3:16:31
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return
	 */
	public static String getUserAccount(String jwt) {
		Claims parseJWT = parseJWT(jwt);
		String userAccount = parseJWT.get(USER_ACCOUNT, String.class);
		return userAccount;
	}
	/**
	 * 
	 * @Description: 获取JWT创建时间
	 *
	 * @date 2018年12月1日,下午3:16:34
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return
	 */
	public static Long getJwtGreatDate (String jwt) {
		Claims parseJWT = parseJWT(jwt);
		Date issuedAt = parseJWT.getIssuedAt();
		Long issuedAtLong = issuedAt.getTime();
		return issuedAtLong;
	}
	
	/**
	 * 
	 * @Description: 校验Jwt是否过期
	 *
	 * @date 2018年12月1日,下午3:44:06
	 * @author tiechanglong
	 * @version 1.2.0
	 *
	 * @param jwt
	 * @return TIME_EXPIRE 过期可续签；TIME_FAILURE失效；TIME_VALID 合法
	 */
	public static JwtStatus checkJwtIsExprie(String jwt) {
		try {
			Claims parseJWT = parseJWT(jwt);
			long time = parseJWT.getIssuedAt().getTime();
			long exprieTime = time + JwtConstant.JWT_VALIDITY;
			Date nowTime = new Date();
			long nowTimeLong = nowTime.getTime();
			if(exprieTime<nowTimeLong) {
				return JwtStatus.TIME_EXPIRE;
			}
		} catch (Exception e) {
			return JwtStatus.TIME_FAILURE;
		}
		return JwtStatus.TIME_VALID;
	}
	
}
